/*
 * $URL$
 * 
 * Copyright (C) 2022 by Arizona State University and Mark Robinson.
 * All rights reserved.
 *
 * $Author$
 *   $Date$
 *    $Rev$
 *
 * (description of this file here)
 *
 */
#include "bias_subtract.h"
#include "special_pixels.h"
#include <stdlib.h>

static int double_spaceship(const void *a, const void *b) {
  if (*(double*)a > *(double*)b) return 1;
  else if (*(double*)a < *(double*)b) return -1;
  else return 0; 
}

void bias_subtract(struct shadowcam_image* image, bool use_median) {
  double (*bias_pile)[SHC_CHANNELS][90000 * 8] = 0;
  bias_pile = calloc(SHC_CHANNELS * 90000 * 8, sizeof(double));

  uint32_t bias_count[SHC_CHANNELS] = {0};

  double bias_median[SHC_CHANNELS] = {0};
  double bias_mean[SHC_CHANNELS] = {0};

  puts("Subtracting per-channel bias pixel average.");

  // using pixel index 2-9 (0-based)
  // put all the non-special bias pixels in a pile
  for (uint32_t line=0; line < image->lines; line++)
    for (uint8_t channel=0; channel < SHC_CHANNELS; channel++)
      for (uint16_t pixel=2; pixel < 10; pixel++)
        if (!is_special(image->data[data_index(line, channel, pixel)]))
          (*bias_pile)[channel][bias_count[channel]++] = image->data[data_index(line, channel, pixel)];

  for (uint8_t channel = 0; channel < SHC_CHANNELS; channel++) {
    // sort the pile
    qsort((*bias_pile)[channel], bias_count[channel], sizeof(double), double_spaceship);

    // find the median
    if (bias_count[channel] % 2 == 0)
      bias_median[channel] = ((double) ((*bias_pile)[channel][bias_count[channel]/2] + (*bias_pile)[channel][bias_count[channel]/2+1])) / 2;
    else
      bias_median[channel] = (double) (*bias_pile)[channel][bias_count[channel/2+1]];

    // find the mean
    for (uint32_t i=0; i<bias_count[channel]; i++)
      bias_mean[channel] += ((double) (*bias_pile)[channel][i])/bias_count[channel];

    printf("  Channel %d bias mean: %f; median: %f\n", channel, bias_mean[channel], bias_median[channel]);
  }

  if (use_median)
    puts("  Using median bias values.");
  else
    puts("  Using mean bias values.");
  
  for (uint32_t line=0; line < image->lines; line++)
    for (uint8_t channel=0; channel < SHC_CHANNELS; channel++)
      for (uint16_t pixel=0; pixel < SHC_AFE_WIDTH; pixel++)
        if (!is_special(image->data[data_index(line, channel, pixel)])) {
          if (use_median)
            image->data[data_index(line, channel, pixel)] = image->data[data_index(line, channel, pixel)] - bias_median[channel];
          else
            image->data[data_index(line, channel, pixel)] = image->data[data_index(line, channel, pixel)] - bias_mean[channel];
        }
}

