/*
 * $URL$
 * 
 * Copyright (C) 2022 by Arizona State University and Mark Robinson.
 * All rights reserved.
 *
 * $Author$
 *   $Date$
 *    $Rev$
 *
 * (description of this file here)
 *
 */
#include "gain_correction.h"
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include "config_path.h"
#include "special_pixels.h"

void gain_correction(struct shadowcam_image *image) {
  puts("Performing gain correction.");

  char *filename = config_file_path("gain_factors.csv");

  FILE *file = fopen(filename, "rb");
  if (file == NULL) {
    printf("fopen of %s failed with errno %d\n", filename, errno);
    exit(1);
  }

  char buffer[1024];
  double gain_factor[SHC_CHANNELS];

  while(fgets(buffer, sizeof(buffer), file)) {
    // skip the header stuff
    if (strncmp(buffer, "#", 1) == 0 || strncmp(buffer, "gain_code", 9) == 0)
      continue;

    double gain_tmp[2 * SHC_CHANNELS];

    char * token = strtok(buffer, ",");
    if (token == NULL) {
      printf("Expected ',' in %s, loading CSV failed.\n", filename);
      exit(1);
    }

    int gain_code = strtol(token, NULL, 10);

    for (int i=0; i< SHC_CHANNELS*2; i++) {
      token = strtok(NULL, ",");
      if (token == NULL) {
        printf("Expected ',' in %s, loading CSV failed.\n", filename);
        exit(1);
      }
      gain_tmp[i] = strtod(token, NULL);
    }

    for (uint8_t channel = 0; channel < SHC_CHANNELS; channel++) {
      if (image->gain[channel] == gain_code) {
        gain_factor[channel] = gain_tmp[channel + SHC_CHANNELS * image->tdi_factor];
        if (gain_factor[channel] == 0) printf("WARNING:");
        printf("  Gain for channel %d: %f\n", channel, gain_factor[channel]);
      }
    }
  }

  free(filename);

  for (uint32_t line=0; line < image->lines; line++)
    for (uint8_t channel=0; channel < SHC_CHANNELS; channel++)
      for (uint16_t pixel=0; pixel < SHC_AFE_WIDTH; pixel++)
        if (!is_special(image->data[data_index(line, channel, pixel)]))
            image->data[data_index(line, channel, pixel)] = image->data[data_index(line, channel, pixel)] / gain_factor[channel];
}
