/*
 * $URL$
 * 
 * Copyright (C) 2022 by Arizona State University and Mark Robinson.
 * All rights reserved.
 *
 * $Author$
 *   $Date$
 *    $Rev$
 *
 * (description of this file here)
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "write_cube.h"
#include <time.h>

void write_cube_suffix(bool keep_bias, const char *filename, struct shadowcam_image *image, const char *suffix) {
  const char *dot = rindex(filename, '.');
  size_t dot_index = dot - filename;
  const char *slash = rindex(filename, '/');
  char *new_name = malloc(strlen(filename) + strlen(suffix) + 2);
  if (new_name == NULL) {
    printf("malloc failed at %s:%d\n", __FILE__, __LINE__);
    exit(1);
  }

  if (slash < dot) {
    strcpy(new_name, filename);
    strcpy(new_name + dot_index, "-");
    strcpy(new_name + dot_index + 1, suffix);
    strcpy(new_name + dot_index + 1 + strlen(suffix), filename + dot_index);
  } else {
    strcpy(new_name, filename);
    strcat(new_name, "-");
    strcat(new_name, suffix);
  }

  write_cube(keep_bias, new_name, image);

  free(new_name);
}

void write_cube(bool keep_bias, const char *filename, struct shadowcam_image *image) {
  printf("Writing to \"%s\"\n", filename);

  uint32_t offset = 65536;

  FILE *file = fopen(filename, "wb");

  uint16_t samples = keep_bias ? image->samples : (SHC_CHANNELS * SHC_SCENE);

  fprintf(file, "Object = IsisCube\n");
  fprintf(file, "  Object = Core\n");
  fprintf(file, "    StartByte = %u\n", offset + 1);
  fprintf(file, "    Format    = BandSequential\n");
  fprintf(file, "\n");
  fprintf(file, "    Group = Dimensions\n");
  fprintf(file, "      Samples = %d\n", samples);
  fprintf(file, "      Lines   = %u\n", image->lines);
  fprintf(file, "      Bands   = 1\n");
  fprintf(file, "    End_Group\n");
  fprintf(file, "\n");
  fprintf(file, "    Group = Pixels\n");
  fprintf(file, "      Type       = Real\n");
  fprintf(file, "      ByteOrder  = Lsb\n");
  fprintf(file, "      Base       = 0.0\n");
  fprintf(file, "      Multiplier = 1.0\n");
  fprintf(file, "    End_Group\n");
  fprintf(file, "  End_Object\n");
  fprintf(file, "\n");
  fprintf(file, "  Group = Instrument\n");
  fprintf(file, "    SpacecraftName                     = \"Korea Pathfinder Lunar Orbiter\"\n");
  fprintf(file, "    InstrumentId                       = \"ShadowCam\"\n");
  fprintf(file, "    TargetName                         = %s\n", image->target_name);
  fprintf(file, "    LineRate                           = %f <ms>\n", image->line_rate_ms);
  fprintf(file, "    LineRateCode                       = %d\n", image->line_rate_code);
  fprintf(file, "    PrerollLines                       = %d\n", image->preroll);
  fprintf(file, "    NumberOfKLines                     = %d\n", image->klines);
  fprintf(file, "    ExecutionTime                      = %s\n", image->execution_time);
  fprintf(file, "    ExecutionSpacecraftTime            = %s\n", image->execution_spacecraft_time);
  fprintf(file, "    StartTimeOffset                    = %f\n", image->start_time_offset);
  fprintf(file, "    PrerollTime                        = %s\n", image->preroll_time);
  fprintf(file, "    StartTime                          = %s\n", image->start_time);
  fprintf(file, "    StopTime                           = %s\n", image->stop_time);
  fprintf(file, "    TemperatureFPAA                    = %f\n", image->fpa_a_temp);
  fprintf(file, "    TemperatureFPAB                    = %f\n", image->fpa_b_temp);
  fprintf(file, "    TemperatureFPGA                    = %f\n", image->fpga_temp);
  fprintf(file, "    TemperatureTelescope               = %f\n", image->telescope_temp);
  fprintf(file, "    Bterm1                             = %d\n", image->bterm[1]);
  fprintf(file, "    Bterm2                             = %d\n", image->bterm[2]);
  fprintf(file, "    Bterm3                             = %d\n", image->bterm[3]);
  fprintf(file, "    Bterm4                             = %d\n", image->bterm[4]);
  fprintf(file, "    Bterm5                             = %d\n", image->bterm[5]);
  fprintf(file, "    Xterm0                             = %d\n", image->xterm[0]);
  fprintf(file, "    Xterm1                             = %d\n", image->xterm[1]);
  fprintf(file, "    Xterm2                             = %d\n", image->xterm[2]);
  fprintf(file, "    Xterm3                             = %d\n", image->xterm[3]);
  fprintf(file, "    Xterm4                             = %d\n", image->xterm[4]);
  fprintf(file, "    ModeTest                           = %s\n", image->mode_test);
  fprintf(file, "    TDIDirection                       = \"%c\"\n", image->tdi_direction);
  fprintf(file, "    GainCh0                            = %d\n", image->gain[0]);
  fprintf(file, "    GainCh1                            = %d\n", image->gain[1]);
  fprintf(file, "    GainCh2                            = %d\n", image->gain[2]);
  fprintf(file, "    GainCh3                            = %d\n", image->gain[3]);
  fprintf(file, "    GainCh4                            = %d\n", image->gain[4]);
  fprintf(file, "    GainCh5                            = %d\n", image->gain[5]);
  fprintf(file, "    OffsetCh0                          = %d\n", image->offset[0]);
  fprintf(file, "    OffsetCh1                          = %d\n", image->offset[1]);
  fprintf(file, "    OffsetCh2                          = %d\n", image->offset[2]);
  fprintf(file, "    OffsetCh3                          = %d\n", image->offset[3]);
  fprintf(file, "    OffsetCh4                          = %d\n", image->offset[4]);
  fprintf(file, "    OffsetCh5                          = %d\n", image->offset[5]);
  fprintf(file, "    BiasSpread                         = %d\n", image->bias_spread);
  fprintf(file, "    BiasSpreadCh0                      = %d\n", image->bias_spread_ch0);
  fprintf(file, "    BiasSpreadCh1                      = %d\n", image->bias_spread_ch1);
  fprintf(file, "    BiasSpreadCh2                      = %d\n", image->bias_spread_ch2);
  fprintf(file, "    BiasSpreadCh3                      = %d\n", image->bias_spread_ch3);
  fprintf(file, "    BiasSpreadCh4                      = %d\n", image->bias_spread_ch4);
  fprintf(file, "    BiasSpreadCh5                      = %d\n", image->bias_spread_ch5);
  fprintf(file, "    SampleFirstPixel                   = %d\n", image->sample_first_pixel);
  fprintf(file, "  End_Group\n");
  fprintf(file, "\n");
  fprintf(file, "  Group = BandBin\n");
  fprintf(file, "    FilterName = BroadBand\n");
  fprintf(file, "    Center     = 600 <nm>\n");
  fprintf(file, "    Width      = 300 <nm>\n");
  fprintf(file, "  End_Group\n");
  fprintf(file, "\n");
  fprintf(file, "  Group = Kernels\n");
  fprintf(file, "    NaifFrameCode = %d\n", image->naif_frame_code);
  fprintf(file, "  End_Group\n");
  fprintf(file, "\n");
  fprintf(file, "  Group = Archive\n");
  fprintf(file, "    ProducerId                         = \"KPLO_SHADOWCAM_TEAM\"\n");
  fprintf(file, "    ProducerInstitutionName            = \"ARIZONA STATE UNIVERSITY\"\n");
  fprintf(file, "    ProductVersion                     = \"v1.1\"\n");
  fprintf(file, "    PDSIdentifier                      = %s\n", image->pds_identifier);
  fprintf(file, "    MissionPhase                       = %s\n", image->mission_phase);
  fprintf(file, "    DataQualityId                      = %d\n", image->dqi);
  fprintf(file, "    DQIFpaOutOfBounds                  = %s\n", image->dqi_fpa_out_of_bounds);
  fprintf(file, "    DQIUnderExopsed                    = %s\n", image->dqi_under_exposed);
  fprintf(file, "    DQIMissingData                     = %s\n", image->dqi_missing_data);
  fprintf(file, "    DQIMissingSpice                    = %s\n", image->dqi_missing_spice);
  fprintf(file, "    DQIUncalibratable                  = %s\n", image->dqi_uncalibratable);
  fprintf(file, "    DQICorruptionDetected              = %s\n", image->dqi_corruption_detected);
  fprintf(file, "    CreationTime                       = ");

  time_t rawtime;
  struct tm * timeinfo;

  time(&rawtime);
  timeinfo = gmtime(&rawtime);
  char time_str[1024];
  strftime(time_str, sizeof(time_str), "%Y-%m-%dT%H:%M:%SZ", timeinfo);
  fprintf(file, "%s\n", time_str);
  fprintf(file, "  End_Group\n");
  fprintf(file, "End_Object\n");
  fprintf(file, "\n");
  fprintf(file, "Object = Label\n");
  fprintf(file, "  Bytes = %u\n", offset);
  fprintf(file, "End_Object\n");
  fprintf(file, "End\n");

  // pad this puppy, hopefully with the right number of bytes
  while(ftell(file) < offset)
    fwrite("\0", 1, 1, file);

  // put data in
  if (keep_bias)
    fwrite(image->data, samples * sizeof(image->data[0]), image->lines, file);
  else
    for(uint32_t line = 0; line < image->lines; line++)
      for(uint8_t channel = 0; channel < SHC_CHANNELS; channel++)
        fwrite(image->data + data_index(line, channel, SHC_SCENE_OFFSET), SHC_SCENE * sizeof(image->data[0]), 1, file);

  fclose(file);
}
